﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using OSGeo.MapGuide;

namespace HelloAIMS2013
{
  public partial class ProfilingAPI : System.Web.UI.Page
  {
    protected void Page_Load(object sender, EventArgs e)
    {
      UtilityClass util = new UtilityClass();
      util.InitializeWebTier(HttpContext.Current.Request);

      String sessionId = Request["SESSION"].ToString();

      util.ConnectToServer(sessionId);
      MgSiteConnection siteConn = util.GetSiteConnection();

      MgResourceService resourceService = siteConn
        .CreateService(MgServiceType.ResourceService) 
        as MgResourceService;
      
      //Create MgProfilingService instance

      MgProfilingService profilingService = siteConn
        .CreateService(MgServiceType.ProfilingService) 
        as MgProfilingService;

      //open current map

      MgMap map = new MgMap();
      if (Request["mapname"] != null)
      {
        string mapName = Request["mapname"].ToString();
        map.Open(resourceService, mapName);

        //remove base layer, profiling API doesn't apply to base layers

        for (int i = 0; i < map.GetLayers().Count; i++)
        {
          MgLayerBase lyr = map.GetLayers().GetItem(i);
          if (lyr.LayerType == MgLayerType.BaseMap)
          {
            map.GetLayers().Remove(lyr);
          }
        }
        map.Save(resourceService);

        #region sample 1, dome of ProfileRenderDynamicOverlay()

        string format = "PNG";
        
        // enum  	{ RenderSelection = 1, RenderLayers = 2, KeepSelection = 4 }
        // Rendering behavior values. 
        //RenderSelection - Renders the selected feature(s). 
        //RenderLayers - Renders the features on the map. 
        //KeepSelection - Renders the selected feature(s) even if they are outside the current scale.

        int behavior = 2;
        MgColor selectionColor = new MgColor(System.Drawing.Color.Blue);
        MgRenderingOptions renderingOptions = new MgRenderingOptions(format,behavior, selectionColor);

        MgByteReader byteReader = profilingService.ProfileRenderDynamicOverlay(
          map, 
          null, //MgSelection, do not have selection here
          renderingOptions);

        //Store in a file
        MgByteSink byteSink = new MgByteSink(byteReader);
        byteSink.ToFile(@"c:\delete\profilingResult.txt");

        Response.Write(@"write result to c:\delete\profilingResult.txt <br/>");

        #endregion

        #region Sample 2, demo of ProfileRenderMap

        MgCoordinate center = map.ViewCenter.Coordinate;
        double scale = map.ViewScale;
        int width = Convert.ToInt32(map.GetDisplayWidth());
        int height = Convert.ToInt32(map.GetDisplayHeight());
        MgColor backgroundColor = new MgColor(map.GetBackgroundColor());
        string format2 = "PNG";
        bool bKeepSelection = true;

        MgByteReader byteReader2 = profilingService.ProfileRenderMap(
          map,
          null,//MgSelection
          center,
          scale,
          width,
          height,
          backgroundColor,
          format2,
          bKeepSelection);

        //Store in a file
        MgByteSink byteSink2 = new MgByteSink(byteReader2);
        byteSink2.ToFile(@"c:\delete\profilingResult2.txt");

        Response.Write(@"write result to c:\delete\profilingResult2.txt<br/>");

        #endregion

      
      }
    }
  }
}