using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
// the above are the defautl reference
// Mapguide reference library
using OSGeo.MapGuide;
using System.Collections;
using System.Text;
using System.Xml;
using System.IO;

public class UtilityClass
{
    // the mapguide connection site object 
    MgSiteConnection siteConnection;

    public MgSiteConnection GetSiteConnection()
    {
        return siteConnection;
    }
    // initialize the web tier
    public void InitializeWebTier(HttpRequest Request)
    {
        string realPath = Request.ServerVariables["APPL_PHYSICAL_PATH"];
        String configPath = realPath + "webconfig.ini";
        MapGuideApi.MgInitializeWebTier(configPath);
    }

    // connect to server
    public void ConnectToServer(String sessionID)
    {
        MgUserInformation userInfo = new MgUserInformation(sessionID);
        siteConnection = new MgSiteConnection();
        siteConnection.Open(userInfo);
    }

    public void ConnectToServer()
    {
        MgUserInformation userInfo = new MgUserInformation("Administrator", "admin");
        siteConnection = new MgSiteConnection();
        siteConnection.Open(userInfo);
    }

    //
    //
    // get the layer by its name 
    //
    //
    public MgLayerBase getLayerByName(MgMap map, String layerName)
    {
        MgLayerBase layer = null;
        for (int i = 0; i < map.GetLayers().GetCount(); i++)
        {
            MgLayerBase nextLayer = map.GetLayers().GetItem(i);
            if (nextLayer.GetName() == layerName)
            {
                layer = nextLayer;
                break;
            }

        }
        return layer;
    }




    //
    //
    // Adds a layer to a layer group. If necessary, it creates the layer group.
    //
    //
    //
    //
    public bool IsDataSourceExists(MgResourceService resourceService, MgResourceIdentifier dataSourceId)
    {
        try
        {
            MgByteReader cnt = resourceService.GetResourceContent(dataSourceId);
            return true;
        }
        catch (MgResourceNotFoundException)
        {
            return false;
        }
    }
    //



    public void CreateFeatureSource(String sessionId, String Srs)
    {
        MgFeatureService featureService = (MgFeatureService)siteConnection.CreateService(MgServiceType.FeatureService);

        // -------Create a feature source with point data.

        // Create a feature class definition for the new feature source

        MgClassDefinition classDefinition = new MgClassDefinition();
        classDefinition.SetName("Points");
        classDefinition.SetDescription("Feature class with point data.");
        classDefinition.SetDefaultGeometryPropertyName("GEOM");

        // Create an identify property
        MgDataPropertyDefinition identityProperty = new MgDataPropertyDefinition("KEY");
        identityProperty.SetDataType(MgPropertyType.Int32);
        identityProperty.SetAutoGeneration(true);
        identityProperty.SetReadOnly(true);

        // Add the identity property to the class definition
        classDefinition.GetIdentityProperties().Add(identityProperty);
        classDefinition.GetProperties().Add(identityProperty);

        // Create a name property
        MgDataPropertyDefinition nameProperty = new MgDataPropertyDefinition("NAME");
        nameProperty.SetDataType(MgPropertyType.String);
        classDefinition.GetProperties().Add(nameProperty);

        // Create a geometry property
        MgGeometricPropertyDefinition geometryProperty = new MgGeometricPropertyDefinition("GEOM");
        geometryProperty.SetGeometryTypes(MgFeatureGeometricType.Point);
        classDefinition.GetProperties().Add(geometryProperty);

        // Create a feature schema
        MgFeatureSchema featureSchema = new MgFeatureSchema("PointSchema", "Point schema");
        featureSchema.GetClasses().Add(classDefinition);


        // Create the feature source
        String featureSourceName = "Session:" + sessionId + "//Points.FeatureSource";
        MgResourceIdentifier resourceIdentifier = new MgResourceIdentifier(featureSourceName);

        //String wkt = "LOCALCS[\"*XY-MT*\",LOCAL_DATUM[\"*X-Y*\",10000],UNIT[\"Meter\", 1],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]";
        MgCreateSdfParams sdfParams = new MgCreateSdfParams("LL84", Srs, featureSchema);

        featureService.CreateFeatureSource(resourceIdentifier, sdfParams);

    }


    void ReleaseReader(MgPropertyCollection res, MgFeatureCommandCollection commands)
    {
        if (res == null)
            return;

        for (int i = 0; i < res.GetCount(); i++)
        {
            MgFeatureCommand cmd = commands.GetItem(i);
            if (cmd is MgInsertFeatures)
            {
                MgFeatureProperty resProp = res.GetItem(i) as MgFeatureProperty;
                if (resProp != null)
                {
                    MgFeatureReader reader = resProp.GetValue() as MgFeatureReader;
                    if (reader == null)
                        return;
                    reader.Close();
                }
            }
        }
    }

    public MgPropertyCollection MakePoint(string name, double x, double y)
    {
        MgPropertyCollection propertyCollection = new MgPropertyCollection();
        MgStringProperty nameProperty = new MgStringProperty("NAME", name);
        propertyCollection.Add(nameProperty);

        MgWktReaderWriter wktReaderWriter = new MgWktReaderWriter();
        MgAgfReaderWriter agfReaderWriter = new MgAgfReaderWriter();

        MgGeometry geometry = wktReaderWriter.Read("POINT XY (" + x.ToString() + " " + y.ToString() + ")");
        MgByteReader geometryByteReader = agfReaderWriter.Write(geometry);
        MgGeometryProperty geometryProperty = new MgGeometryProperty("GEOM", geometryByteReader);
        propertyCollection.Add(geometryProperty);

        return propertyCollection;
    }

    // Adds a layer defition (which can be stored either in the Library or a session
    // repository) to the map.
    // Returns the layer.
    public MgLayer add_layer_resource_to_map(MgResourceIdentifier layerResourceID, MgResourceService resourceService, string layerName, string layerLegendLabel, MgMap map)
    {
        MgLayer newLayer = new MgLayer(layerResourceID, resourceService);

        // Add the new layer to the map's layer collection
        newLayer.SetName(layerName);
        newLayer.SetVisible(true);					// *** NFG?
        newLayer.SetLegendLabel(layerLegendLabel);
        newLayer.SetDisplayInLegend(true);
        MgLayerCollection layerCollection = map.GetLayers();
        if (!layerCollection.Contains(layerName))
        {
            // Insert the new layer at position 0 so it is at the top
            // of the drawing order
            layerCollection.Insert(0, newLayer);
        }
        newLayer.SetDisplayInLegend(true);
        return newLayer;
    }



    // Adds a layer to a layer group. If necessary, it creates the layer group.
    public void add_layer_to_group(MgLayerBase layer, string layerGroupName, string layerGroupLegendLabel, MgMap map)
    {

        // Get the layer group
        MgLayerGroupCollection layerGroupCollection = map.GetLayerGroups();
        MgLayerGroup layerGroup = null;
        if (layerGroupCollection.Contains(layerGroupName))
        {
            layerGroup = layerGroupCollection.GetItem(layerGroupName);
        }
        else
        {
            // It does not exist, so create it
            layerGroup = new MgLayerGroup(layerGroupName);
            layerGroup.SetVisible(true);
            layerGroup.SetDisplayInLegend(true);
            layerGroup.SetLegendLabel(layerGroupLegendLabel);
            layerGroupCollection.Add(layerGroup);
        }
        layerGroup.SetDisplayInLegend(true);
        // Add the layer to the group
        layer.SetGroup(layerGroup);

    }


    public String GetMapSrs(MgMap map)
    {
        String srs = map.GetMapSRS();
        if (srs != "")
            return srs;

        //SRS is currently optional. Waiting for this to change, set the default SRS to ArbitrayXY meters
        //
        return "LOCALCS[\"*XY-MT*\",LOCAL_DATUM[\"*X-Y*\",10000],UNIT[\"Meter\", 1],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH]]";
    }

    public UtilityClass()
    {
    }

    public bool CreatePointsLayer(String rootPath, String sessionId)
    {
        MgResourceService resourceService = (MgResourceService)siteConnection.CreateService(MgServiceType.ResourceService);
        MgFeatureService featureService = (MgFeatureService)siteConnection.CreateService(MgServiceType.FeatureService);

        // Open the map
        MgMap map = new MgMap();
        map.Open(resourceService, "Sheboygan");
        String srs = map.GetMapSRS();

        // -------Create a feature source with point data
        CreateFeatureSource(sessionId, srs);

        String featureSourceName = "Session:" + sessionId + "//Points.FeatureSource";
        MgResourceIdentifier resourceIdentifier = new MgResourceIdentifier(featureSourceName);

        MgBatchPropertyCollection batchPropertyCollection = new MgBatchPropertyCollection();
        MgWktReaderWriter wktReaderWriter = new MgWktReaderWriter();
        MgAgfReaderWriter agfReaderWriter = new MgAgfReaderWriter();
        MgGeometryFactory geometryFactory = new MgGeometryFactory();

        // Make four points 	
        MgPropertyCollection propertyCollection = MakePoint("Point A", -87.727, 43.748);
        batchPropertyCollection.Add(propertyCollection);

        propertyCollection = MakePoint("Point B", -87.728, 43.730);
        batchPropertyCollection.Add(propertyCollection);

        propertyCollection = MakePoint("Point C", -87.726, 43.750);
        batchPropertyCollection.Add(propertyCollection);

        propertyCollection = MakePoint("Point D", -87.728, 43.750);
        batchPropertyCollection.Add(propertyCollection);

        // Add the batch property collection to the feature source
        MgInsertFeatures cmd = new MgInsertFeatures("Points", batchPropertyCollection);
        MgFeatureCommandCollection featureCommandCollection = new MgFeatureCommandCollection();
        featureCommandCollection.Add(cmd);

        // Execute the "add" commands
        featureService.UpdateFeatures(resourceIdentifier, featureCommandCollection, false);
        MgResourceIdentifier resourceID = null;

        //Please type in the code between the comments
        // -------Create a new layer
        LayerDefinitionFactory factory = new LayerDefinitionFactory();
        factory.RootDirectoryPath = rootPath;

        // Create a mark symbol
        string resourceSymbel = "Library://Samples/Sheboygan/Symbols/BasicSymbols.SymbolLibrary";
        string symbolName = "PushPin";
        string width = "24";  // unit = points
        string height = "24"; // unit = points
        string color = "FFFF0000";
        string markSymbol = factory.CreateMarkSymbol(resourceSymbel, symbolName, width, height, color);

        // Create a text symbol
        string text = "ID";
        string fontHeight = "12";
        string foregroundColor = "FF000000";
        string textSymbol = factory.CreateTextSymbol(text, fontHeight, foregroundColor);

        // Create a point rule.
        string legendLabel = "trees";
        string filter = "";
        string pointRule = factory.CreatePointRule(legendLabel, filter, textSymbol, markSymbol);

        // Create a point type style.
        string pointTypeStyle = factory.CreatePointTypeStyle(pointRule);

        // Create a scale range.
        string minScale = "0";
        string maxScale = "1000000000000";
        string pointScaleRange = factory.CreateScaleRange(minScale, maxScale, pointTypeStyle);

        // Create the layer definiton.
        string featureName = "PointSchema:Points";
        string geometry = "GEOM";
        string layerDefinition = factory.CreateLayerDefinition(featureSourceName, featureName, geometry, pointScaleRange);

        //Create/Modify layer definition
        MgByteSource byteSource = new MgByteSource(Encoding.UTF8.GetBytes(layerDefinition), layerDefinition.Length);
        MgByteReader byteReader = byteSource.GetReader();


        resourceID = new MgResourceIdentifier("Session:" + sessionId + "//Points.LayerDefinition");
        resourceService.SetResource(resourceID, byteReader, null);
        //////////////////////////////////////////////////////////////////////////

        MgByteSink byteSink = new MgByteSink(resourceService.GetResourceContent(resourceID));
        string filePath = rootPath + "\\Supported\\LayerDefinition.xml";
        byteSink.ToFile(filePath);

        MgLayer newLayer = add_layer_resource_to_map(resourceID, resourceService, "Points", "Points legen", map);

        add_layer_to_group(newLayer, "Analysis", "Analysis", map);

        MgLayerCollection layerCollection = map.GetLayers();
        if (layerCollection.Contains("Points"))
        {
            MgLayerBase pointsLayer = layerCollection.GetItem("Points");
            pointsLayer.SetVisible(true);
        }
        map.Save(resourceService);

        return true;
    }

    public string plotMap(string rootPath)
    {
        MgResourceService resourceService = (MgResourceService)siteConnection.CreateService(MgServiceType.ResourceService);
        MgMappingService mappingService = (MgMappingService)siteConnection.CreateService(MgServiceType.MappingService);
        MgMap map = new MgMap();
        map.Open(resourceService, "Sheboygan");

        //Type in the code between the comments
        MgDwfVersion dwfVer = new MgDwfVersion("6.01", "1.2");
        MgPlotSpecification plotSpec = new MgPlotSpecification(8.5f, 11.0f, MgPageUnitsType.Inches, 0f, 0f, 0f, 0f);
        plotSpec.SetMargins(0.5f, 0.5f, 0.5f, 0.5f);

        MgResourceIdentifier layoutRes = new MgResourceIdentifier("Library://Samples/Sheboygan/Layouts/SheboyganMap.PrintLayout");
        MgLayout layout = new MgLayout(layoutRes, "City of Sheboygan", MgPageUnitsType.Inches);

        MgCoordinate mapCenter = map.GetViewCenter().GetCoordinate();
        double scale = map.GetViewScale();
        MgByteReader byteReader = mappingService.GeneratePlot(map, mapCenter, scale, plotSpec, layout, dwfVer);
        //MgByteReader byteReader = mappingService.GeneratePlot(map, plotSpec, layout, dwfVer);

        MgByteSink byteSink = new MgByteSink(byteReader);
        string filePath = rootPath +  "Supported\\Map.DWF";
        byteSink.ToFile(filePath);
        //////////////////////////////////////////////

        return filePath;
    }

    public string createMapImage(string rootPath)
    {
        //Type in the code between the comments
        MgResourceService resourceService = (MgResourceService)siteConnection.CreateService(MgServiceType.ResourceService);
        MgRenderingService renderingService = (MgRenderingService)siteConnection.CreateService(MgServiceType.RenderingService);

        MgMap map = new MgMap();
        map.Open(resourceService, "Sheboygan");
        int with = map.DisplayWidth;

        MgSelection selection = new MgSelection(map);
        selection.Open(resourceService, "Sheboygan");

        MgEnvelope extent = map.GetMapExtent();
        double x = extent.GetLowerLeftCoordinate().GetX();
        double y = extent.GetLowerLeftCoordinate().GetY();

        MgColor color = new MgColor("FFFFBF20");

        MgByteReader byteReader = renderingService.RenderMap(map, selection, extent, 450, 613, color, "PNG");


        MgByteSink byteSink = new MgByteSink(byteReader);
        string filePath = rootPath + "Supported\\Map.PNG";
        byteSink.ToFile(filePath);

        /////////////////////////////////////////

        return filePath;
    }

    public String DescribeDrawing()
    {
        String result = "";

        MgDrawingService drawingService = (MgDrawingService)siteConnection.CreateService(MgServiceType.DrawingService);
        MgResourceIdentifier drawing_ID = new MgResourceIdentifier("Library://Exercise/Data/BlueHarbor.DrawingSource");
        MgByteReader reader = drawingService.DescribeDrawing(drawing_ID);
        result = reader.ToString();

        return result;
    }
}
